/*
This program has been ported to Turbo C++ for Windows by John C. Wright
from the original Macintosh program, ERGO, by Rob Goldston.
	Rob Goldston						John C. Wright
	125 Clover Lane                     Princeton Plasma Physics Lab   		
	Princeton, New Jersey 08540         Princeton, NJ 08540                            	
	(609) 921-1462                      (609) 243-3789
    goldston@pppl.gov					jwright@pppl.gov

This program was compiled under Borland's Turbo C++ for Windows 3.1
Required files for compilation:
	ergo.prg	Project File
	ergo.rc     Resource file (specifies menus, etc.)
	rc_ids.h    Resource ID numbers
	main.cpp    main program (this file)
    
Other files:
	ergo.wri	Instructions for Ergo in Write format
	ergo.txt	In plain text w/o screen shots
    ergo.exe	Windows 3.1 executable
*/

#include <owl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <edit.h>
#include <radiobut.h>
#include <scrollba.h>
#include <static.h>
#include "rc_ids.h"


#define PI2 6.283185307
#define WW 500
#define WH 400
#define DEFAULT_STEPS 100 			//plot interval in milli-seconds
#define D_DELAY_STRING " pts/sec" 	//same value as above but in quotes
#define TIMER_DELAY  1000U
#define MAX_STEPS 500
#define SUBHEIGHT 20 				//off set to exclude coordinates and 
									//speedbar from plotting area

#define GMAP_INIT FALSE            	//initial values for dialogue box
#define CMAP_INIT TRUE              //for mapping parameters
#define EPS_INIT  "0.1"
#define THBP_INIT  "3.14"
#define THB0_INIT "0.0"
#define THPP_INIT "1.0"
#define THP0_INIT "0.0"
#define RMIN_INIT "-10.0"
#define RMAX_INIT "10.0"
#define N_INIT     "1"

BOOL CMAP=CMAP_INIT;

struct TTransferStruct {
	BOOL gmapState;
	BOOL cmapState;
	char eps[6];
	char thbp[6];
	char thb0[6];
	char thpp[6];
	char thp0[6];
    char rmax[6];
	char rmin[6];
	char n[6];
};

const COLORREF Colors[8] = {RGB(255,0,0),RGB(0,255,0),RGB(0,0,255),
							RGB(255,128,255),RGB(237,237,0),RGB(0,0,0),
							RGB(242,148,34),RGB(220,0,240)};                            							 
int color_index = 0;

// Declare TheApp, a TApplication descendant
class TheApp : public TApplication
{
public:
  TheApp(LPSTR AName, HINSTANCE hInstance, HINSTANCE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
    : TApplication(AName, hInstance, hPrevInstance, lpCmdLine, nCmdShow) {};
  virtual void InitMainWindow();
  friend class TErgoWindow;
};

// Declare TErgoWindow, a TWindow descendant
class TErgoWindow : public TWindow
{
	double eps,thbp,thb0,thpp,thp0,rmin,rmax;
	int n;
    BOOL canmap,gomap;
	char *endptr;
	float r,th;
	int window_width,window_height;
	int steps;
	TScrollBar *SpeedBar;
	TStatic *SpeedStatic;
	TStatic *RPos;
	TStatic *ThetaPos;
  	HBITMAP scrnbm;
public:
	TTransferStruct ErgoTransfer;
	TErgoWindow(PTWindowsObject AParent, LPSTR ATitle);
    ~TErgoWindow();
 	virtual BOOL CanClose();
 	virtual void Paint(HDC PaintDC, PAINTSTRUCT _FAR& PaintInfo);
	virtual void GetWindowClass( WNDCLASS& WndClass );
    virtual void SetupWindow();
	virtual void CMSeed() = [CM_FIRST + CM_SEED];    //functions to handle menus
	virtual void CMDone() = [CM_FIRST + CM_DONE];      
	virtual void CMMap() = [CM_FIRST + CM_MAP];
	virtual void CMStopMap() = [CM_FIRST + CM_STOPMAP];
	virtual void WMLButtonDown(RTMessage Msg) = [WM_FIRST + WM_LBUTTONDOWN];  //mouse fns
	virtual void WMMouseMove(RTMessage Msg) = [WM_FIRST + WM_MOUSEMOVE];
	virtual void UpdateDelay() = [ID_FIRST + ID_SPEEDBAR]; //function for slider control
	void draw_trajectory();        //function that calculates map and plots points
	void SetupScreenBitmap();      //setup copy of  drawing area for redrawing 
	virtual void WMTimer() = [WM_FIRST + WM_TIMER]; //handle timing for different speeds
};

// Declare ErgoDlg, a TDialog Decendent to allow input of Ergo's map properties
class ErgoDlg : public TDialog {
public:
	ErgoDlg(PTWindowsObject AParent, int ResourceId);
	virtual void WMInitDialog(RTMessage Msg) = [WM_FIRST + WM_INITDIALOG]; //add to member function
	virtual void HideTHB0(RTMessage Msg) = [ID_FIRST + IDB_CMAP]; //Process CMAP button
	virtual void ShowTHB0(RTMessage Msg) = [ID_FIRST + IDB_GMAP]; //Process GMAP button
	virtual BOOL CanClose();//Process Quit message
};



//	----------------Begin ErgoDlg Implementations

ErgoDlg::ErgoDlg(PTWindowsObject AParent, int ResourceId)
		: TDialog(AParent, ResourceId)
{
	new TRadioButton(this, IDB_GMAP, NULL);  //these declarations must be in
	new TRadioButton(this, IDB_CMAP, NULL);  //the same order as the variables
	new TEdit(this, ID_eps,                  //in the transfer structure
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.eps));  
	new TEdit(this, ID_thbp,
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.thbp));
	new TEdit(this, ID_thb0,
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.thb0));
	new TEdit(this, ID_thpp,
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.thpp));
	new TEdit(this, ID_thp0,
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.thp0));
	new TEdit(this, ID_rmax,
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.rmax));
	new TEdit(this, ID_rmin,
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.rmin));
	new TEdit(this, ID_n,
		sizeof(((TErgoWindow *)Parent)->ErgoTransfer.n));
	TransferBuffer = (void far*)
					 &(((TErgoWindow *)Parent)->ErgoTransfer);
}

void ErgoDlg::WMInitDialog(RTMessage Msg)//initialize dialogue with two TEDITs
{                                        //faded if CMAP is set
    TDialog::WMInitDialog(Msg);
	if (CMAP) {
	EnableWindow(GetDlgItem(HWindow, ID_thb0), FALSE);
	EnableWindow(GetDlgItem(HWindow, ID_thbp), FALSE);
	}
}

void ErgoDlg::HideTHB0(RTMessage)
{
	EnableWindow(GetDlgItem(HWindow, ID_thb0), FALSE);
	EnableWindow(GetDlgItem(HWindow, ID_thbp), FALSE);
}

void ErgoDlg::ShowTHB0(RTMessage)
{
	EnableWindow(GetDlgItem(HWindow, ID_thb0), TRUE);
	EnableWindow(GetDlgItem(HWindow, ID_thbp), TRUE);
}
 
BOOL ErgoDlg::CanClose()
{
	return 1;
}


// ---------------------------Begin TErgoWindow Implementations


/* Constructor for a TErgoWindow, sets styles and
   constructs the Window object. */
TErgoWindow::TErgoWindow(PTWindowsObject AParent, LPSTR ATitle)
		 : TWindow(AParent, ATitle)
{
	char tmp[15];
	sprintf(tmp,"%d pts/sec",(1000/TIMER_DELAY)*DEFAULT_STEPS);
	AssignMenu("ERGO_MENU");
	memset(&ErgoTransfer, 0x0, sizeof ErgoTransfer);
	Attr.Style = WS_CAPTION | WS_SYSMENU |WS_MINIMIZEBOX ;
	Attr.X = 20;
	Attr.Y = 40;
	Attr.W = WW;
	Attr.H = WH;
	steps = DEFAULT_STEPS;
	rmin = -10.0;
    rmax = 10.0;
	SpeedBar = new TScrollBar(this, ID_SPEEDBAR, Attr.W-110, 1, 100, 18,TRUE);
	SpeedStatic = new TStatic(this, ID_SPEEDSTATIC, tmp, Attr.W-230, 1,
								100, 17, 0);
	RPos = new TStatic(this, ID_RPos, "0", 5, 1, 70, 17, 0);
	ThetaPos = new TStatic(this, ID_ThetaPos, "0", 85, 1, 80, 17, 0);
//	Initialize the transfer structure
	ErgoTransfer.gmapState = GMAP_INIT;
	ErgoTransfer.cmapState = CMAP_INIT;
	strcpy(ErgoTransfer.eps,EPS_INIT);
	strcpy(ErgoTransfer.thbp,THBP_INIT);
	strcpy(ErgoTransfer.thb0,THB0_INIT);
	strcpy(ErgoTransfer.thpp,THPP_INIT);
	strcpy(ErgoTransfer.thp0,THP0_INIT);
    strcpy(ErgoTransfer.rmax,RMAX_INIT);
	strcpy(ErgoTransfer.rmin,RMIN_INIT);
	strcpy(ErgoTransfer.n,N_INIT);

	canmap=FALSE;
	gomap=FALSE;
	r = (rmax+rmin)/2.0;
    th = 0.0;
}

void TErgoWindow::SetupWindow()
{
	TWindow::SetupWindow();		//Default class function

	SpeedBar->SetRange(1,MAX_STEPS);     //setup speed scrollbar controller
	SpeedBar->LineMagnitude = 1;
	SpeedBar->PageMagnitude = 10;
	SpeedBar->SetPosition(DEFAULT_STEPS);

	RECT client_rect;
	GetClientRect(HWindow, &client_rect);
	window_width = client_rect.right;
	window_height = client_rect.bottom-SUBHEIGHT;


//setup timer for controlling drawing speed
	if (!SetTimer(HWindow, ID_MYTIMER, TIMER_DELAY, NULL))      
		MessageBox(HWindow, "Too Many Timers", "Ergo",
					MB_ICONEXCLAMATION | MB_OK);
	SetupScreenBitmap();
}

void TErgoWindow::SetupScreenBitmap()
{
//Setup virtual screen bitmap for redrawing when screen is uncovered
	HDC hdc,hmemdc;
	PAINTSTRUCT PtStr;
    HBRUSH hbr,holdbr;

	hdc = BeginPaint(HWindow, &PtStr);
	hmemdc = CreateCompatibleDC(hdc);

	scrnbm = CreateCompatibleBitmap(hdc, Attr.W, Attr.H);
	SelectObject(hmemdc, scrnbm);


	hbr = CreateSolidBrush(RGB(255,255,255));
	holdbr = (HBRUSH) SelectObject(hmemdc, hbr);
	PatBlt(hmemdc,0,0,Attr.W,Attr.H,PATCOPY);

	MoveTo(hmemdc,0,SUBHEIGHT);
    LineTo(hmemdc,Attr.W,SUBHEIGHT);

	SelectObject(hmemdc, holdbr);
	EndPaint(HWindow, &PtStr);
	DeleteObject(holdbr);
	DeleteDC(hmemdc);
}

TErgoWindow::~TErgoWindow()
{
	KillTimer(HWindow, ID_MYTIMER);
	DeleteObject(scrnbm);
	delete SpeedBar;
	delete SpeedStatic;
	delete RPos;
	delete ThetaPos;
}

void  TErgoWindow::GetWindowClass( WNDCLASS& WndClass)
{
	TWindow::GetWindowClass( WndClass );
	WndClass.hIcon  = LoadIcon( GetApplication()->hInstance, "ERGO_ICON" );
	WndClass.hCursor = LoadCursor( GetApplication()->hInstance, "ERGO_CURSOR");
}

void TErgoWindow::CMSeed()
{
	gomap = FALSE;
	char Ergodata[80], *endptr;
	if ( GetModule()->ExecDialog(new ErgoDlg(this,ID_DIALOG)) == IDOK ){
	eps = strtod(ErgoTransfer.eps, &endptr);
	thbp = strtod(ErgoTransfer.thbp, &endptr);
	thb0 = strtod(ErgoTransfer.thb0, &endptr);
	thpp = strtod(ErgoTransfer.thpp, &endptr);
	thp0 = strtod(ErgoTransfer.thp0, &endptr);
	rmax = strtod(ErgoTransfer.rmax, &endptr);
	rmin = strtod(ErgoTransfer.rmin, &endptr);
	n = atoi(ErgoTransfer.n);
	CMAP = ErgoTransfer.cmapState;
	if (CMAP) SetCaption("Ergo: Chirikov-Taylor Map");
    	else  SetCaption("Ergo: Goldston-White-Boozer Map");
	canmap = TRUE;  //allow mapping now that user has set parameters
//  Erase virtual screen bitmap and send Paint message to redraw window
    DeleteObject(scrnbm);
	SetupScreenBitmap();
    InvalidateRect(HWindow, NULL, TRUE);
	}
}

void TErgoWindow::CMStopMap()
{
	gomap = FALSE;
}

void TErgoWindow::CMDone()
{	
	CloseWindow();
}


void TErgoWindow::CMMap()
{
	if (!canmap) MessageBox(HWindow,
	  "Please be sure to set the plot parameters first.","Error!",MB_OK);
	else gomap=canmap;
}

void TErgoWindow :: WMLButtonDown(RTMessage Msg)   //Initialize trajectory
{                                                  //When left mouse button
	WORD x,y;                                      //is pressed
  
    x = Msg.LP.Lo;     
	y = Msg.LP.Hi-SUBHEIGHT;

	r = rmax - (rmax - rmin) * (y+0.5)/window_height;
	th = PI2 * (x+0.5)/window_width;
	gomap = canmap;
	color_index = (int)(color_index++);
	if (color_index==6) color_index=0;
    WMTimer();
}

BOOL TErgoWindow::CanClose()
{
	BOOL value;
	gomap = FALSE;
	value = (BOOL) MessageBox(HWindow, "", "Sure you want to quit?"
		, MB_YESNO | MB_ICONQUESTION) == IDYES;
	if (!value) gomap = canmap;
    return value;
}


// Responds to an incoming "paint" message by redrawing screen.
void TErgoWindow::Paint(HDC PaintDC,PAINTSTRUCT& PaintInfo)
{
	HDC hmemdc;

	hmemdc = CreateCompatibleDC(PaintDC);
	SelectObject(hmemdc, scrnbm);
	BitBlt(PaintDC, 0, 0, Attr.W, Attr.H, hmemdc, 0, 0, SRCCOPY);
	DeleteDC(hmemdc);
}

void TErgoWindow::WMTimer()   //when timer is triggered, draw #points determined
{                             //by speedbar setting
	int i,j;
	MSG msg;

	if (gomap)
		{
		for (i=0; i<steps; i++)
			{
			draw_trajectory();
			j+=1;
			if (j>50)
				{
				j=0;
				if ( PeekMessage(&msg, NULL, (WM_FIRST + WM_LBUTTONDOWN),
					(WM_FIRST + WM_LBUTTONDOWN), PM_REMOVE) )
					  WMLButtonDown((RTMessage)msg);
				else if ( PeekMessage(&msg, NULL, (WM_FIRST + WM_MOUSEMOVE),
					(WM_FIRST + WM_MOUSEMOVE), PM_REMOVE) )
					 { WMMouseMove((RTMessage)msg); }
//                       if () 
				else if (PeekMessage(&msg, NULL,(CM_FIRST+101),(CM_FIRST+110),
					PM_NOREMOVE)) return;
				}
        	}
		}
}

void TErgoWindow::UpdateDelay()
{
  char AString[15];

  steps = SpeedBar->GetPosition();
  sprintf(AString, "%d pts/sec", (1000/TIMER_DELAY)*steps);
  SpeedStatic->SetText(AString);
}

void TErgoWindow::WMMouseMove(RTMessage Msg)
{
	WORD x,y;
  
    x = Msg.LP.Lo;     
	y = Msg.LP.Hi-SUBHEIGHT;

    float rcoor, thcoor;
	char rpos[15];
	char thpos[15];

    if ((int)y >= 0 ) {
	rcoor = rmax - (rmax - rmin) * (y+0.5)/window_height;
	thcoor = PI2 * (x)/window_width;

	sprintf(rpos,"r: %6.2f",rcoor);
	sprintf(thpos,"th: %6.2f",thcoor);

	RPos->SetText(rpos);
	ThetaPos->SetText(thpos); }        
}

void TErgoWindow::draw_trajectory()
{
	HDC hDC, hmemdc;
    HPEN hpenOld,hpen;
	float rplot,thplot;

	hDC = GetDC(HWindow);
	hmemdc = CreateCompatibleDC(hDC);
	SelectObject(hmemdc, scrnbm);

        if (ErgoTransfer.cmapState) {
        	th = th + thp0 + thpp*r;
			r  = r + eps*cos(n*th);
			}
		else {
			th = th + thp0 + thpp*r + thb0 + thbp*r;
			r  = r + eps*cos(n*th);
			th = th + thp0 + thpp*r - thb0 - thbp*r;
			r  = r + eps*cos(n*th);
            }
        th = fmod(th,PI2);
        if (th < 0.0) th = th + PI2;                                
        if (r > rmin && r < rmax) {  
			rplot = SUBHEIGHT+window_height*((rmax-r)/(rmax-rmin));
			thplot = window_width*(th/PI2);
            hpen = CreatePen(PS_SOLID,1, Colors[color_index]);
			hpenOld = (HPEN) SelectObject(hmemdc, hpen);
			MoveTo(hmemdc,thplot,rplot);
			LineTo(hmemdc,thplot+1,rplot);
			SetPixel(hDC, thplot, rplot, Colors[color_index]);
			SelectObject(hmemdc,hpenOld);
			DeleteObject(hpen);
			}
        else MessageBeep(-1);
	ReleaseDC(HWindow, hDC);
    DeleteDC(hmemdc);
}


//--------------------Begin TheApp definitions
void TheApp::InitMainWindow()
{
  MainWindow = new TErgoWindow(NULL, "Ergo: No map chosen.");
}

// Run TheApp
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
  LPSTR lpCmdLine, int nCmdShow)
{
  TheApp ERGO("Ergo", hInstance, hPrevInstance, lpCmdLine, nCmdShow);
  ERGO.Run();
  return ERGO.Status;
}
